@@ -32,6 +32,10 @@ import threading
 import websocket
 from copy import deepcopy
 
+
+appendIpAddress = "192.168.1.1"
+# appendIpAddress = None
+
 # REAL ENV.
 KAISA_ROOT_URL = "kgl.jt00000.com"
 KAISA_ROOT_URL_SIM = "sit-kgl.jt00000.com"
@@ -47,7 +51,6 @@ QUOTE_URL = "https://kgl.jt00000.com/hq"
 WEBSOCKET_DATA_HOST = "ws://sit-kgl.jt00000.com/dz_app_ws/ws"
 REST_DATA_HOST = "https://__kaisarooturl__/dzApp/dzHttpApi"
 
-
 TRADE_CRYPTO_RSA_URL = "https://__kaisarooturl__/kgl-trade-service/crypto/key/RSA"
 ClientByMobile_URL = "https://__kaisarooturl__/kgl-user-center/userOaccAccount/selectClientIdByMobile"
 REST_HOST = "https://__kaisarooturl__/kgl-trade-service"
@@ -55,12 +58,16 @@ WEBSOCKET_TRADE_HOST = "wss://__kaisarooturl__/kgl-trade-push-service/ws"
 
 openapi_scope = "wthk"
 
-# from common import KaisaCrypto
-# from common.utils import *
+MARKET_POST_URL = "https://sit-kgl.jt00000.com/kgl-stock-push-provider"
+MARKET_WS_URL = "ws://uat-stock.jt00000.com/stock"
+
 from kaisaglobalapi.common import KaisaCrypto
 from kaisaglobalapi.common.utils import *
 
+from kaisaglobalapi.common import KaisaProtoc
+
 kcrypto = KaisaCrypto()
+kprotoc = KaisaProtoc()
 
 START_PUSH = 200
 STOP_PUSH = 201
@@ -104,6 +111,10 @@ class KaisaGateway():
     _market_ws = None
     _trade_ws = None
 
+    _market_connect_status = False
+
+    _ws_trade_connect_status = False
+
     def __init__(self):
         pass
 
@@ -144,7 +155,6 @@ class KaisaGateway():
     def write_log(self, data):
         print("log: {}".format(data))
 
-
     def _authentica(self, auth_username: str, auth_password: str) -> None:
         """
         获取网关登录令牌;
@@ -169,16 +179,25 @@ class KaisaGateway():
             "grant_type": "password",
             "scope": openapi_scope
         }
+
+        if appendIpAddress:
+            params["ipAddress"] = appendIpAddress
+
+        headerParam = {"ipAddress":"14.154.204.168","phoneModel":"","deviceId":"","deviceToken":"","macAddress":"","channelId":"","country":"","sysVersion":"","apiVersion":"","platform":""}
+        headerParam = json.dumps(headerParam)
         headers = {
             "Authorization": "basic {}".format(self.openapi_token),
             "Content-Type": "application/x-www-form-urlencoded",
             "Sign": sign_,
-            "Timestamp": timestamp_
+            "Timestamp": timestamp_,
+            "headerParam": headerParam
         }
         response = requests.post(
             url=self.AUTHENTICA_URL, params=params, headers=headers
         )
 
+        self.gateway_auth_msg = ""
+
         data = response.json()
         if response.status_code // 100 == 2:
             self.write_log("网关认证成功")
@@ -197,13 +216,15 @@ class KaisaGateway():
             self.write_log("网关认证失败")
             self.write_error(data)
 
-        return self.gateway_auth_status
+
+        return self.gateway_auth_status, self.gateway_auth_msg
 
     def gateway_auth(self):
         return self._authentica(self.user_id, self.user_pwd)
 
-    def market_connect(self):
-        if self.gateway_auth():
+    def market_connect_ws(self):
+        ret_ok, ret_msg = self.gateway_auth()
+        if ret_ok:
             self.market_connect_ws()
         time.sleep(5)
 
@@ -301,65 +322,171 @@ class KaisaGateway():
         return symbols
 
 
+    def query_history_kline(self, symbol='01638', market='HK', klinetype='1d', adjfactortype='0', pagenum=1, pagesize=100):
+        """
+        query history kline (only for hk market)
+        """
+        post_url = MARKET_POST_URL+"/quoteInfo/kLine"
+
+        klinetype2klineid = {'1min': 1,
+                             '5min': 3,
+                             '15min': 5,
+                             '30min': 6,
+                             '1hour': 7,
+                             '1day': 10
+                             }
+        klineid = klinetype2klineid.get(klinetype)
+        rqdata = {
+            'code': symbol,
+            'market': market,
+            'quoteTypeEnumCode': klineid,
+            'adjFactorType': adjfactortype,
+            'pageNum': pagenum,
+            'pageSize': pagesize
+        }
+        response = requests.request(
+            'POST',
+            post_url,
+            headers={'Content-Type':'application/json'},
+            params=None,
+            data=json.dumps(rqdata)
+        )
+        status_code = response.status_code
+        resp_data = response.json()
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']['records']
+        else:
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
+
+
+    def query_history_tick(self, symbol='00700', market='HK', pagesize=10, pagenum=1):
+
+        post_url = MARKET_POST_URL + "/stock/quote/tick"
+        rqdata = {
+            'code': symbol,
+            'market': market,
+            'size': str(pagesize),
+            'index': str(pagesize*pagenum)
+        }
+        response = requests.request(
+            'POST',
+            post_url,
+            headers={'Content-Type':'application/json'},
+            params=None,
+            data=json.dumps(rqdata)
+        )
+        status_code = response.status_code
+        resp_data = response.json()
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
+        else:
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
+
+
+    def query_detailstate(self, symbol='01638', market='HK'):
+
+        post_url = MARKET_POST_URL+ "/quoteInfo/todayDetailState"
+        rqdata = {
+            'code': symbol,
+            'market': market,
+            'hasQuoteInfo': True,
+            'hasQuoteExt': True,
+            'hasQuoteStat': True,
+            'hasQuoteFinancial': True,
+            'hasQuotePro': True
+        }
+        response = requests.request(
+            'POST',
+            post_url,
+            headers={'Content-Type': 'application/json', 'headerParam':json.dumps({'apiVersion':"1.2.0"})},
+            params=None,
+            data=json.dumps(rqdata)
+        )
+        status_code = response.status_code
+        resp_data = response.json()
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
+            del body_data['assetInfoResp']
+        else:
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
+
+
     def market_connect_ws(self):
+
         threading.Thread(target=self.__market_connect_ws, daemon=True).start()
 
+    def get_market_connect_ws_status(self):
 
-    def get_market_connect_status(self):
-
-        return self._market_connect_status
+        return self._market_connect_status, self._market_connect_msg
 
     def __market_connect_ws(self):
 
-        header = {"Authorization": self.gatewayToken}
+        if self._market_connect_status:
+            return
 
-        self._market_ws = websocket.create_connection(self.WEBSOCKET_DATA_HOST, header=header)
+        header = {"Authorization": self.gatewayToken,
+                  "deviceId":"xxx",
+                  "platform":"open"}
+
+        self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
         self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
         self._market_ping_th.start()
 
         self._market_connect_status = True
+        self._market_connect_msg = ""
 
+        print('wait to receive.')
         while True:
+
             text = self._market_ws.recv()
-            if len(text)==0:
+            data = kprotoc.rec_bstring_to_json(text)
+            if data is None:
                 continue
-            data = json.loads(text)
-            self.on_market_packet(data)
+            self.on_market_data(data)
+
 
     def on_market_packet(self, packet):
 
-        # print("on kaisa market data:")
-        # print(packet)
-
         self.on_market_data(packet)
 
     def on_market_data(self, data):
         pass
 
-    def subscribe_marketdata(self, symbols = ["00700"]):
+    def subscribe_marketdata(self, code_list):
         """
-        推送行情和分笔;
-        :param symbols:
-        :return:
+        推送行情和分笔；
         """
-        data = [{"market": 2002, "code": symbol, "type": 3, "language": 0} for symbol in symbols]
-        req = self.generate_req(200, data)
-        self.send_market_packet(req)
+        self.req_id += 1
+        bstring = kprotoc.get_subscribe_bstring(code_list, self.req_id)
+        self._market_ws.send_binary(bstring)
 
-    def unsubscribe_marketdata(self, symbols = ["00700"]):
+    def unsubscribe_marketdata(self, code_list):
         """
         取消推送行情和分笔
         :param symbols:
         :return:
         """
-        data = [{"market": 2002, "code": symbol, "type": 3, "language": 0} for symbol in symbols]
-        req = self.generate_req(201, data)
-        self.send_market_packet(req)
+        self.req_id+=1
+        bstring = kprotoc.get_unsubscribe_bstring(code_list, self.req_id)
+        self._market_ws.send_binary(bstring)
 
     def market_heartbeat(self):
-        data = {}
-        req = self.generate_req(3, data)
-        self.send_market_packet(req)
+
+        self.req_id += 1
+        ping_bstring = kprotoc.get_ping_bstring(self.req_id)
+        self._market_ws.send_binary(ping_bstring)
 
     def send_market_packet(self, packet):
 
@@ -568,8 +695,9 @@ class KaisaGateway():
             status_code, resp_data = self.do_trade_requests(url, reqdata, "connect")
             self.sessionId = resp_data['body']['sessionId']
             self.trade_connect_status = True
-        # self.start_trade_ws()
-        return self.trade_connect_status
+
+        self.trade_connect_msg = ''
+        return self.trade_connect_status, self.trade_connect_msg
 
 
     def trade_auth(self):
@@ -592,14 +720,15 @@ class KaisaGateway():
         else:
             self.tradeToken = None
             self.trade_auth_status = False
-        return self.trade_auth_status
+        self.trade_auth_msg = ''
+        return self.trade_auth_status, self.trade_auth_msg
 
 
-    def start_trade_ws(self):
+    def trade_connect_ws(self):
         threading.Thread(target=self._start_trade_ws, daemon=True).start()
 
-    def get_trade_ws_connect_status(self):
-        return self._ws_trade_connect_status
+    def get_trade_connect_ws_status(self):
+        return self._ws_trade_connect_status, ""
 
     def _market_ping(self, this_ws):
         """
@@ -607,7 +736,6 @@ class KaisaGateway():
         :return:
         """
         while True:
-            this_ws.send("ping", websocket.ABNF.OPCODE_PING)
             self.market_heartbeat()
             time.sleep(5)
 
@@ -617,12 +745,13 @@ class KaisaGateway():
         :return:
         """
         pass
-        # while True:
-        #     # this_ws.send("ping", websocket.ABNF.OPCODE_PING)
-        #     time.sleep(5)
+
 
     def _start_trade_ws(self):
 
+        if self._ws_trade_connect_status:
+            return True
+
         #
         data = {
             "channelType": "INTERNET",
@@ -638,19 +767,12 @@ class KaisaGateway():
 
         self._trade_ws = websocket.create_connection(host, header=header)
 
-        # many_symbols_partial = ["00700"]
-        # data = [{"market": 2002, "code": code, "type": 1, "language": 0} for code in many_symbols_partial]
-
-        # if not self.trade_login_status:
         req = self.generate_req(LOGIN, data)
-        # self.send_packet(req)
+
         req = json.dumps(req)
         self._trade_ws.send(req)
-
         self._trade_ping_th = threading.Thread(target=self._trade_ping, args=(self._trade_ws,), daemon=True)
         self._trade_ping_th.start()
-        # time.sleep(3)
-
         self._ws_trade_connect_status = True
 
         while True:
@@ -662,11 +784,7 @@ class KaisaGateway():
 
         if len(packet)==0:
             return
-        # print("rev")
-        # print(type(packet))
-        # print(packet)
         data = json.loads(packet)
-
         if data.get('reqtype',0)==2:
             req_data = {"ts":data['data']["ts"]}
             req_data = self.generate_req(PONG, req_data)
@@ -678,7 +796,7 @@ class KaisaGateway():
 
     def send_order(self, bsFlag='B', price=550, qty=600, code='00700'):
 
-        self.place_order(bsFlag='B', price=550, qty=600, code='00700')
+        return self.place_order(bsFlag=bsFlag, price=price, qty=qty, code=code)
 
     def place_order(self, bsFlag='B', price=550, qty=600, code='00700'):
 
@@ -696,12 +814,15 @@ class KaisaGateway():
         url = self.REST_HOST+"/v1/order/orders/place"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
 
-        # print(resp_data)
-        if status_code//100==2:
-            rec_data = resp_data['body']
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            rec_data = None
-        return rec_data
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
+
 
     def cancel_order(self, sys_orderid):
 
@@ -716,12 +837,16 @@ class KaisaGateway():
 
         url = self.REST_HOST+"/v1/order/orders/cancel"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
-        if status_code//100==2:
-            # rec_data = resp_data['body']
-            rec_data = resp_data
+
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            rec_data = None
-        return rec_data
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
+
 
     def query_portfolio(self):
 
@@ -730,11 +855,15 @@ class KaisaGateway():
 
         url = self.REST_HOST+"/v1/account/accounts/portfolio"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
-        if status_code//100==2:
-            resp_data = resp_data['body']
+
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            resp_data = None
-        return resp_data
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
 
 
     def query_balance(self):
@@ -743,11 +872,15 @@ class KaisaGateway():
                    "params": {'accountCode': self.account_code}}
         url = self.REST_HOST + "/v1/account/accounts/balance"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
-        if status_code//100==2:
-            resp_data = resp_data['body']
+        resp_data = resp_data.json()
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            resp_data = None
-        return resp_data
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
 
 
     def query_position(self):
@@ -756,11 +889,15 @@ class KaisaGateway():
                    "params": {'accountCode': self.account_code}}
         url = self.REST_HOST + "/v1/account/accounts/position"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
-        if status_code//100==2:
-            resp_data = resp_data['body']
+
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            resp_data = None
-        return resp_data
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
 
 
     def query_order(self):
@@ -769,86 +906,15 @@ class KaisaGateway():
                    "params": {'accountCode': self.account_code}}
         url = self.REST_HOST + "/v1/order/orders"
         status_code, resp_data = self.do_trade_requests(url, reqdata)
-        if status_code//100==2:
-            resp_data = resp_data['body']
+
+        resp_status = resp_data['success']
+        if resp_status:
+            body_data = resp_data['body']
         else:
-            resp_data = None
-        return resp_data
-
-
-    def query_symboldata(self, symbols):
-        """
-        请求市场数据
-        :return:
-        """
-        data = {
-            "reqtype": 153,
-            "data": {
-                "getsyminfo": 1,
-                "symbol": [{"market":2002, "code":symbol} for symbol in symbols]
-            }
-        }
-        url = self.REST_DATA_HOST
-        reqdata = {"data": data
-                   }
-        status_code, resp_data = self.do_market_requests(url, reqdata)
-
-        if status_code//100==2:
-            rec_data = resp_data['data']['symbol']
-        else:
-            rec_data = None
-        return rec_data
-
-    def query_klinedata(self, symbol, start_time, end_time, weight, klinetype):
-        """
-        请求历史数据
-        :param start_time:
-        :param end_time:
-        :return:
-        :weight: 0表示不复权，1表示前复权，2表示后复权
-        """
-        klinetypedict = {
-            "1m": 1,
-            "5m": 2,
-            "60m": 3,
-            "3m": 4,
-            "15m": 5,
-            "30m": 6,
-            "120m": 7,
-            "1d": 10,
-            "1w": 11,
-            "m": 20,
-            "q": 21,
-            "y": 30
-        }
-
-        klinetypevalue = klinetypedict.get(klinetype,0)
-
-        query_dict = {
-            "market": 2002,
-            "code": symbol,
-            "klinetype": klinetypevalue,
-            "weight": weight,
-            "timetype": 0,
-            "time0": start_time.strftime("%Y-%m-%d %H:%M:%S"),
-            "time1": end_time.strftime("%Y-%m-%d %H:%M:%S"),
-            "count": 10000
-        }
-        self.req_id += 1
-        data = {
-            "reqtype": 150,
-            "reqid": self.req_id,
-            "session": "",
-            "data": query_dict
-        }
-        reqdata = {"data": data
-                   }
-        url = self.REST_DATA_HOST
-
-        status_code, resp_data = self.do_market_requests(url, reqdata)
-        resp_data = resp_data['data']['kline']
-        return resp_data
-
+            body_data = {'retCode': resp_data['retCode'],
+                         'retMsg': resp_data['retMsg']
+                         }
+        return resp_status, body_data
 
 if __name__=="__main__":
 
